@@ -29,8 +29,13 @@
 # include "config.h"
 #endif
 
-#include "blowfish.h"
-#include "endianness.h"
+#include "beecrypt/blowfish.h"
+
+#if HAVE_ENDIAN_H && HAVE_ASM_BYTEORDER_H
+# include <endian.h>
+#endif
+
+#include "beecrypt/endianness.h"
 
 #ifdef ASM_BLOWFISHENCRYPTECB
 extern int blowfishEncryptECB(blowfishparam*, uint32_t*, const uint32_t*, unsigned int);
