@@ -14,8 +14,8 @@
 #define PyObject_HEAD   int _PyObjectHead;
 #endif
 
-#include "mpw-py.h"
-#include "rng-py.h"
+#include "beecrypt/python/mpw-py.h"
+#include "beecrypt/python/rng-py.h"
 
 #include "debug-py.c"
 
